{# Plyr video player styles #}
<link rel="stylesheet" href="https://cdn.plyr.io/3.7.8/plyr.css" />

{# Styles to make the Plyr video fit the container #}
<style lang="css">
    .video-container {
        width: 100%;
        height: 450px;
    }

    .plyr {
        height: 100%;
        width: 100%;
    }

    .plyr__video-wrapper {
        height: 100%;
    }

    .plyr__video-wrapper iframe {
        width: 100%;
        height: 100%;
    }
</style>
